@@ -100,7 +100,7 @@ MDAnalysis_. Some tools help in building and interpreting index files.
 .. autofunction:: make_ndx_captured
 
 
-.. _MDAnalysis: http://mdanalysis.googlecode.com
+.. _MDAnalysis: http://mdanalysis.org
 .. _VMD: http://www.ks.uiuc.edu/Research/vmd/current/ug/node87.html
 .. _Charmm: http://www.charmm.org/html/documentation/c35b1/select.html
 
@@ -125,7 +125,7 @@ file. For specialized cases the two following functions are useful:
 # new class and set arguments explicitly in init (using kwargs['flag'] =
 # default) ... or I can write some meta(??) class to do this nicely
 
-from __future__ import with_statement
+from __future__ import absolute_import, with_statement
 
 __docformat__ = "restructuredtext en"
 
@@ -136,15 +136,16 @@ import warnings
 import tempfile
 import shutil
 import glob
+import six
 
 import logging
 logger = logging.getLogger('gromacs.cbook')
 
 import gromacs
-from gromacs import GromacsError, BadParameterWarning, MissingDataWarning, GromacsValueWarning, GromacsImportWarning
-import tools
-import utilities
-from utilities import asiterable
+from .exceptions import GromacsError, BadParameterWarning, MissingDataWarning, GromacsValueWarning, GromacsImportWarning
+from . import tools
+from . import utilities
+from .utilities import asiterable
 
 def _define_canned_commands():
     """Define functions for the top level name space.
@@ -199,12 +200,12 @@ one to fit.
 
 try:
     _define_canned_commands()
-except (OSError, ImportError, GromacsError):
-    warnings.warn("Failed to define a number of commands in gromacs.cbook. Most likely the "
-                  "Gromacs installation cannot be found --- source GMXRC!",
-                  category=GromacsImportWarning)
-    logger.error("Failed to define a number of commands in gromacs.cbook. Most likely the "
-                  "Gromacs installation cannot be found --- source GMXRC!")
+except (OSError, ImportError, AttributeError, GromacsError):
+    msg = ("Failed to define a number of commands in gromacs.cbook. Most "
+          "likely the Gromacs installation cannot be found --- set GMXRC in "
+           "~/.gromacswrapper.cfg or source GMXRC directly")
+    warnings.warn(msg, category=GromacsImportWarning)
+    logger.error(msg)
 finally:
     del _define_canned_commands
 
@@ -224,8 +225,7 @@ def trj_fitandcenter(xy=False, **kwargs):
        *o*
            output trajectory
        *input*
-           A list with three groups. The default is
-               ['backbone', 'protein','system']
+           A list with three groups. The default is ``['backbone', 'protein','system']``.
            The fit command uses all three (1st for least square fit,
            2nd for centering, 3rd for output), the centered/make-whole stage use
            2nd for centering and 3rd for output.
@@ -316,11 +316,11 @@ def trj_fitandcenter(xy=False, **kwargs):
 
     fd, tmptrj = tempfile.mkstemp(suffix=suffix, prefix='pbc_compact_')
 
-    logger.info("Input structure for PBC:  %(compact_structure)r" % vars())
-    logger.info("Input structure for fit:  %(fit_structure)r" % vars())
-    logger.info("Input trajectory:  %(intrj)r" % vars())
-    logger.info("Output trajectory: %(outtrj)r"% vars())
-    logger.debug("Writing temporary trajectory %(tmptrj)r (will be auto-cleaned)." % vars())
+    logger.info("Input structure for PBC:  {compact_structure!r}".format(**vars()))
+    logger.info("Input structure for fit:  {fit_structure!r}".format(**vars()))
+    logger.info("Input trajectory:  {intrj!r}".format(**vars()))
+    logger.info("Output trajectory: {outtrj!r}".format(**vars()))
+    logger.debug("Writing temporary trajectory {tmptrj!r} (will be auto-cleaned).".format(**vars()))
     sys.stdout.flush()
     try:
         gromacs.trjconv(s=compact_structure, f=intrj, o=tmptrj, n=ndxcompact,
@@ -370,13 +370,15 @@ def cat(prefix="md", dirname=os.path.curdir, partsdir="parts", fulldir="full",
     :Keywords:
         *prefix*
             deffnm of the trajectories [md]
-        *resolve_multi"
+        *resolve_multi*
             how to deal with multiple "final" gro or pdb files: normally there should
             only be one but in case of restarting from the checkpoint of a finished
             simulation one can end up with multiple identical ones.
+
               - "pass" : do nothing and log a warning
               - "guess" : take prefix.pdb or prefix.gro if it exists, otherwise the one of
-                          prefix.partNNNN.gro|pdb with the highes NNNN
+                prefix.partNNNN.gro|pdb with the highes NNNN
+
         *dirname*
             change to *dirname* and assume all tarjectories are located there [.]
         *partsdir*
@@ -402,7 +404,7 @@ def cat(prefix="md", dirname=os.path.curdir, partsdir="parts", fulldir="full",
         nonempty_files = []
         for f in filenames:
             if os.stat(f).st_size == 0:
-                logger.warn("File %(f)r is empty, skipping" % vars())
+                logger.warn("File {f!r} is empty, skipping".format(**vars()))
                 continue
             if os.path.islink(f):
                 # TODO: re-write the symlink to point to the original file
@@ -516,7 +518,7 @@ class Frames(object):
         self.structure = structure  # tpr or equivalent
         self.trj = trj              # xtc, trr, ...
         self.maxframes = maxframes
-        if not self.maxframes is None:
+        if self.maxframes is not None:
             raise NotImplementedError('sorry, maxframes feature not implemented yet')
 
         self.framedir = tempfile.mkdtemp(prefix="Frames_", suffix='_'+format)
@@ -573,7 +575,7 @@ class Frames(object):
         self.framedir = None
 
     def __del__(self):
-        if not self.framedir is None:
+        if self.framedir is not None:
             self.cleanup()
 
 # Working with topologies
@@ -582,11 +584,11 @@ class Frames(object):
 # grompp that does not raise an exception; setting up runs the command to get the docs so
 # we only want to do this once at the module level and not inside a function that can be called
 # repeatedly
-grompp_warnonly = tools.Grompp(failure="warn",
-                               doc="grompp wrapper that only warns on failure but does not raise :exc:`GromacsError`")
+grompp_warnonly = tools.Grompp(failure="warn")
+# grompp_warnonly.__doc__ += "\n\ngrompp wrapper that only warns on failure but does not raise :exc:`GromacsError`"
 
 def grompp_qtot(*args, **kwargs):
-    """Run ``gromacs.grompp`` and return the total charge of the  system.
+    r"""Run ``gromacs.grompp`` and return the total charge of the  system.
 
     :Arguments:
        The arguments are the ones one would pass to :func:`gromacs.grompp`.
@@ -610,7 +612,7 @@ def grompp_qtot(*args, **kwargs):
       :regexp:`System has non-zero total charge: *(?P<qtot>[-+]?\d*\.\d+([eE][-+]\d+)?)`.
 
     """
-    qtot_pattern = re.compile('System has non-zero total charge: *(?P<qtot>[-+]?\d*\.\d+([eE][-+]\d+)?)')
+    qtot_pattern = re.compile(r"System has non-zero total charge: *(?P<qtot>[-+]?\d*\.\d+([eE][-+]\d+)?)")
     # make sure to capture ALL output
     kwargs['stdout'] = False
     kwargs['stderr'] = False
@@ -632,7 +634,7 @@ def grompp_qtot(*args, **kwargs):
         if m:
             qtot = float(m.group('qtot'))
             break
-    logger.info("system total charge qtot = %(qtot)r" % vars())
+    logger.info("system total charge qtot = {qtot!r}".format(**vars()))
     return qtot
 
 def _mdp_include_string(dirs):
@@ -724,7 +726,7 @@ def create_portable_topology(topol, struct, **kwargs):
       *topol*
           topology file
       *struct*
-          coordinat (structure) file
+          coordinate (structure) file
 
     :Keywords:
       *processed*
@@ -743,8 +745,8 @@ def create_portable_topology(topol, struct, **kwargs):
     processed = kwargs.pop('processed', os.path.join(_topoldir, 'pp_'+_topol))
     grompp_kwargs, mdp_kwargs = filter_grompp_options(**kwargs)
     mdp_kwargs = add_mdp_includes(topol, mdp_kwargs)
-    with tempfile.NamedTemporaryFile(suffix='.mdp') as mdp:
-        mdp.write('; empty mdp file\ninclude = %(include)s\n' % mdp_kwargs)
+    with tempfile.NamedTemporaryFile(suffix='.mdp', mode='wb') as mdp:
+        mdp.write('; empty mdp file\ninclude = {include!s}\n'.format(**mdp_kwargs).encode('utf-8'))
         mdp.flush()
         grompp_kwargs['p'] = topol
         grompp_kwargs['pp'] = processed
@@ -775,7 +777,7 @@ def get_volume(f):
 # ---------------------------
 
 def edit_mdp(mdp, new_mdp=None, extend_parameters=None, **substitutions):
-    """Change values in a Gromacs mdp file.
+    r"""Change values in a Gromacs mdp file.
 
     Parameters and values are supplied as substitutions, eg ``nsteps=1000``.
 
@@ -840,61 +842,62 @@ def edit_mdp(mdp, new_mdp=None, extend_parameters=None, **substitutions):
         extend_parameters = list(asiterable(extend_parameters))
 
     # None parameters should be ignored (simple way to keep the template defaults)
-    substitutions = dict([(k,v) for k,v in substitutions.items() if not v is None])
+    substitutions = {k: v for k,v in substitutions.items() if v is not None}
 
-    params = substitutions.keys()[:]   # list will be reduced for each match
+    params = list(substitutions.keys())   # list will be reduced for each match
 
     def demangled(p):
         """Return a RE string that matches the parameter."""
         return p.replace('_', '[-_]')  # must catch either - or _
 
-    patterns = dict([(parameter,
-                      re.compile("""\
-                       (?P<assignment>\s*%s\s*=\s*)  # parameter == everything before the value
+    patterns = {parameter:
+                      re.compile(r"""
+                       (?P<assignment>\s*{0!s}\s*=\s*)  # parameter == everything before the value
                        (?P<value>[^;]*)              # value (stop before comment=;)
                        (?P<comment>\s*;.*)?          # optional comment
-                       """ % demangled(parameter), re.VERBOSE))
-                     for parameter in substitutions])
+                       """.format(demangled(parameter)), re.VERBOSE)
+                     for parameter in substitutions}
+
+    with tempfile.TemporaryFile() as target:
+        with open(mdp, 'rb') as src:
+            logger.info("editing mdp = {0!r}: {1!r}".format(mdp, substitutions.keys()))
+            for line in src:
+                line = line.decode('utf-8')
+                new_line = line.strip()  # \n must be stripped to ensure that new line is built without break
+                for p in params[:]:
+                    m = patterns[p].match(new_line)
+                    if m:
+                        # I am too stupid to replace a specific region in the string so I rebuild it
+                        # (matching a line and then replacing value requires TWO re calls)
+                        #print 'line:' + new_line
+                        #print m.groupdict()
+                        if m.group('comment') is None:
+                            comment = ''
+                        else:
+                            comment = " "+m.group('comment')
+                        assignment = m.group('assignment')
+                        if not assignment.endswith(' '):
+                            assignment += ' '
+                        # build new line piece-wise:
+                        new_line = assignment
+                        if p in extend_parameters:
+                            # keep original value and add new stuff at end
+                            new_line += str(m.group('value')) + ' '
+                        # automatically transform lists into space-separated string values
+                        value = " ".join(map(str, asiterable(substitutions[p])))
+                        new_line += value + comment
+                        params.remove(p)
+                        break
+                target.write((new_line+'\n').encode('utf-8'))
+        target.seek(0)
+        # XXX: Is there a danger of corrupting the original mdp if something went wrong?
+        with open(new_mdp, 'wb') as final:
+            shutil.copyfileobj(target, final)
 
-    target = tempfile.TemporaryFile()
-    with open(mdp) as src:
-        logger.info("editing mdp = %r: %r" % (mdp, substitutions.keys()))
-        for line in src:
-            new_line = line.strip()  # \n must be stripped to ensure that new line is built without break
-            for p in params[:]:
-                m = patterns[p].match(new_line)
-                if m:
-                    # I am too stupid to replace a specific region in the string so I rebuild it
-                    # (matching a line and then replacing value requires TWO re calls)
-                    #print 'line:' + new_line
-                    #print m.groupdict()
-                    if m.group('comment') is None:
-                        comment = ''
-                    else:
-                        comment = " "+m.group('comment')
-                    assignment = m.group('assignment')
-                    if not assignment.endswith(' '):
-                        assignment += ' '
-                    # build new line piece-wise:
-                    new_line = assignment
-                    if p in extend_parameters:
-                        # keep original value and add new stuff at end
-                        new_line += str(m.group('value')) + ' '
-                    # automatically transform lists into space-separated string values
-                    value = " ".join(map(str, asiterable(substitutions[p])))
-                    new_line += value + comment
-                    params.remove(p)
-                    break
-            target.write(new_line+'\n')
-    target.seek(0)
-    # XXX: Is there a danger of corrupting the original mdp if something went wrong?
-    with open(new_mdp, 'w') as final:
-        shutil.copyfileobj(target, final)
-    target.close()
      # return all parameters that have NOT been substituted
     if len(params) > 0:
-        logger.warn("Not substituted in %(new_mdp)r: %(params)r" % vars())
-    return dict([(p, substitutions[p]) for p in params])
+        logger.warn("Not substituted in {new_mdp!r}: {params!r}".format(**vars()))
+    return {p: substitutions[p] for p in params}
 
 def edit_txt(filename, substitutions, newname=None):
     """Primitive text file stream editor.
@@ -953,35 +956,35 @@ def edit_txt(filename, substitutions, newname=None):
     _substitutions = [{'lRE': re.compile(str(lRE)),
                        'sRE': re.compile(str(sRE)),
                        'repl': repl}
-                      for lRE,sRE,repl in substitutions if not repl is None]
+                      for lRE,sRE,repl in substitutions if repl is not None]
 
-    target = tempfile.TemporaryFile()
-    with open(filename) as src:
-        logger.info("editing txt = %r (%d substitutions)" % (filename, len(substitutions)))
-        for line in src:
-            keep_line = True
-            for subst in _substitutions:
-                m = subst['lRE'].match(line)
-                if m:              # apply substition to this line?
-                    logger.debug('match:    '+line.rstrip())
-                    if subst['repl'] is False:   # special rule: delete line
-                        keep_line = False
-                    else:                        # standard replacement
-                        line = subst['sRE'].sub(str(subst['repl']), line)
-                        logger.debug('replaced: '+line.rstrip())
-            if keep_line:
-                target.write(line)
-            else:
-                logger.debug("Deleting line %r", line)
+    with tempfile.TemporaryFile() as target:
+        with open(filename, 'rb') as src:
+            logger.info("editing txt = {0!r} ({1:d} substitutions)".format(filename, len(substitutions)))
+            for line in src:
+                line = line.decode("utf-8")
+                keep_line = True
+                for subst in _substitutions:
+                    m = subst['lRE'].match(line)
+                    if m:              # apply substition to this line?
+                        logger.debug('match:    '+line.rstrip())
+                        if subst['repl'] is False:   # special rule: delete line
+                            keep_line = False
+                        else:                        # standard replacement
+                            line = subst['sRE'].sub(str(subst['repl']), line)
+                            logger.debug('replaced: '+line.rstrip())
+                if keep_line:
+                    target.write(line.encode('utf-8'))
+                else:
+                    logger.debug("Deleting line %r", line)
 
-    target.seek(0L)
-    with open(newname, 'w') as final:
-        shutil.copyfileobj(target, final)
-    target.close()
-    logger.info("edited txt = %(newname)r" % vars())
+        target.seek(0)
+        with open(newname, 'wb') as final:
+            shutil.copyfileobj(target, final)
+    logger.info("edited txt = {newname!r}".format(**vars()))
 
 def remove_molecules_from_topology(filename, **kwargs):
-    """Remove autogenerated [ molecules ] entries from *filename*.
+    r"""Remove autogenerated [ molecules ] entries from *filename*.
 
     Valid entries in ``[ molecules ]`` below the default *marker*
     are removed. For example, a topology file such as ::
@@ -1026,26 +1029,26 @@ def remove_molecules_from_topology(filename, **kwargs):
         marker = "; Gromacs auto-generated entries follow:"
     logger.debug("Scrubbed [ molecules ]: marker = %(marker)r", vars())
 
-    p_marker = re.compile("\s*%s" % marker)
-    p_molecule = re.compile("\s*[\w+_-]+\s+\d+\s*(;.*)?$")
-    target = tempfile.TemporaryFile()
-    with open(filename) as src:
-        autogenerated = False
-        n_removed = 0
-        for line in src:
-            if p_marker.match(line):
-                autogenerated = True
-            if autogenerated and p_molecule.match(line):
-                n_removed += 1
-                continue  # remove by skipping
-            target.write(line)
-    if autogenerated and n_removed > 0:
-        target.seek(0L)
-        with open(filename, 'w') as final:   # overwrite original!
-            shutil.copyfileobj(target, final)
-        logger.info("Removed %(n_removed)d autogenerated [ molecules ] from "
-                    "topol = %(filename)r" % vars())
-    target.close()
+    p_marker = re.compile(r"\s*{0!s}".format(marker))
+    p_molecule = re.compile(r"\s*[\w+_-]+\s+\d+\s*(;.*)?$")
+    with tempfile.TemporaryFile() as target:
+        with open(filename, 'rb') as src:
+            autogenerated = False
+            n_removed = 0
+            for line in src:
+                line = line.decode('utf-8')
+                if p_marker.match(line):
+                    autogenerated = True
+                if autogenerated and p_molecule.match(line):
+                    n_removed += 1
+                    continue  # remove by skipping
+                target.write(line.encode('utf-8'))
+        if autogenerated and n_removed > 0:
+            target.seek(0)
+            with open(filename, 'wb') as final:   # overwrite original!
+                shutil.copyfileobj(target, final)
+            logger.info("Removed %(n_removed)d autogenerated [ molecules ] from "
+                        "topol = %(filename)r" % vars())
     return n_removed
 
 
@@ -1323,7 +1326,7 @@ class IndexBuilder(object):
         except (TypeError, IndexError):
             gmx_resid = resid + self.offset
         except KeyError:
-            raise KeyError("offset must be a dict that contains the gmx resid for %d" % resid)
+            raise KeyError("offset must be a dict that contains the gmx resid for {0:d}".format(resid))
         return gmx_resid
 
     def combine(self, name_all=None, out_ndx=None, operation='|', defaultgroups=False):
@@ -1360,11 +1363,10 @@ class IndexBuilder(object):
         .. SeeAlso:: :meth:`IndexBuilder.write`.
         """
         if not operation in ('|', '&', False):
-            raise ValueError("Illegal operation %r, only '|' (OR) and '&' (AND) or False allowed." %
-                             operation)
-        if name_all is None:
-            if operation:
-                name_all = self.name_all or operation.join(self.indexfiles)
+            raise ValueError("Illegal operation {0!r}, only '|' (OR) and '&' (AND) or False allowed.".format(
+                             operation))
+        if name_all is None and operation:
+            name_all = self.name_all or operation.join(self.indexfiles)
         if out_ndx is None:
             out_ndx = self.output
 
@@ -1388,18 +1390,18 @@ class IndexBuilder(object):
                 fd, tmp_ndx = tempfile.mkstemp(suffix='.ndx', prefix='combined__')
                 # combine all selections by loading ALL temporary index files
                 operation = ' '+operation.strip()+' '
-                cmd = [operation.join(['"%s"' % gname for gname in self.indexfiles]),
+                cmd = [operation.join(['"{0!s}"'.format(gname) for gname in self.indexfiles]),
                        '', 'q']
                 rc,out,err = self.make_ndx(n=ndxfiles, o=tmp_ndx, input=cmd)
                 if self._is_empty_group(out):
-                    warnings.warn("No atoms found for %(cmd)r" % vars(),
+                    warnings.warn("No atoms found for {cmd!r}".format(**vars()),
                                   category=BadParameterWarning)
 
                 # second pass for naming, sigh (or: use NDX ?)
                 groups = parse_ndxlist(out)
                 last = groups[-1]
                 # name this group
-                name_cmd = ["name %d %s" % (last['nr'], name_all), 'q']
+                name_cmd = ["name {0:d} {1!s}".format(last['nr'], name_all), 'q']
                 rc,out,err = self.make_ndx(n=tmp_ndx, o=out_ndx, input=name_cmd)
                 # For debugging, look at out and err or set stdout=True, stderr=True
                 # TODO: check out if at least 1 atom selected
@@ -1456,7 +1458,7 @@ class IndexBuilder(object):
 
         self._command_counter += 1
         if name is None:
-            name = "CMD%03d" % self._command_counter
+            name = "CMD{0:03d}".format(self._command_counter)
 
         # Need to build it with two make_ndx calls because I cannot reliably
         # name the new group without knowing its number.
@@ -1465,7 +1467,7 @@ class IndexBuilder(object):
             cmd = [command, '', 'q']   # empty command '' necessary to get list
             # This sometimes fails with 'OSError: Broken Pipe' --- hard to debug
             rc,out,err = self.make_ndx(o=tmp_ndx, input=cmd)
-            self.check_output(out, "No atoms found for selection %(command)r." % vars(), err=err)
+            self.check_output(out, "No atoms found for selection {command!r}.".format(**vars()), err=err)
             # For debugging, look at out and err or set stdout=True, stderr=True
             # TODO: check '  0 r_300_&_ALA_&_O     :     1 atoms' has at least 1 atom
             ##print "DEBUG: _process_command()"
@@ -1474,8 +1476,8 @@ class IndexBuilder(object):
             last = groups[-1]
             # reduce and name this group
             fd, ndx = tempfile.mkstemp(suffix='.ndx', prefix=name+'__')
-            name_cmd = ["keep %d" % last['nr'],
-                        "name 0 %s" % name, 'q']
+            name_cmd = ["keep {0:d}".format(last['nr']),
+                        "name 0 {0!s}".format(name), 'q']
             rc,out,err = self.make_ndx(n=tmp_ndx, o=ndx, input=name_cmd)
         finally:
             utilities.unlink_gmx(tmp_ndx)
@@ -1483,7 +1485,7 @@ class IndexBuilder(object):
         return name, ndx
 
     #: regular expression to match and parse a residue-atom selection
-    RESIDUE = re.compile("""
+    RESIDUE = re.compile(r"""
                  (?P<aa>([ACDEFGHIKLMNPQRSTVWY])   # 1-letter amino acid
                         |                          #   or
                         ([A-Z][A-Z][A-Z][A-Z]?)    # 3-letter or 4-letter residue name
@@ -1503,7 +1505,7 @@ class IndexBuilder(object):
         # XXX: use _translate_residue() ....
         m = self.RESIDUE.match(selection)
         if not m:
-            raise ValueError("Selection %(selection)r is not valid." % vars())
+            raise ValueError("Selection {selection!r} is not valid.".format(**vars()))
 
         gmx_resid = self.gmx_resid(int(m.group('resid')))
         residue = m.group('aa')
@@ -1516,10 +1518,10 @@ class IndexBuilder(object):
             gmx_atomname = 'CA'
 
         #: select residue <gmx_resname><gmx_resid> atom <gmx_atomname>
-        _selection = 'r %(gmx_resid)d & r %(gmx_resname)s & a %(gmx_atomname)s' % vars()
+        _selection = 'r {gmx_resid:d} & r {gmx_resname!s} & a {gmx_atomname!s}'.format(**vars())
         cmd = ['keep 0', 'del 0',
                _selection,
-               'name 0 %(name)s' % vars(),
+               'name 0 {name!s}'.format(**vars()),
                'q']
         fd, ndx = tempfile.mkstemp(suffix='.ndx', prefix=name+'__')
         rc,out,err = self.make_ndx(n=self.ndx, o=ndx, input=cmd)
@@ -1550,15 +1552,15 @@ class IndexBuilder(object):
                 logger.error("%r is not a valid range selection", selection)
                 raise
         if name is None:
-            name = "%(first)s-%(last)s_%(gmx_atomname)s" % vars()
+            name = "{first!s}-{last!s}_{gmx_atomname!s}".format(**vars())
 
         _first = self._translate_residue(first, default_atomname=gmx_atomname)
         _last = self._translate_residue(last, default_atomname=gmx_atomname)
 
-        _selection = 'r %d - %d & & a %s' % (_first['resid'],  _last['resid'], gmx_atomname)
+        _selection = 'r {0:d} - {1:d} & & a {2!s}'.format(_first['resid'], _last['resid'], gmx_atomname)
         cmd = ['keep 0', 'del 0',
                _selection,
-               'name 0 %(name)s' % vars(),
+               'name 0 {name!s}'.format(**vars()),
                'q']
         fd, ndx = tempfile.mkstemp(suffix='.ndx', prefix=name+'__')
         rc,out,err = self.make_ndx(n=self.ndx, o=ndx, input=cmd)
@@ -1575,7 +1577,7 @@ class IndexBuilder(object):
         """Translate selection for a single res to make_ndx syntax."""
         m = self.RESIDUE.match(selection)
         if not m:
-            errmsg = "Selection %(selection)r is not valid." % vars()
+            errmsg = "Selection {selection!r} is not valid.".format(**vars())
             logger.error(errmsg)
             raise ValueError(errmsg)
 
@@ -1606,8 +1608,7 @@ class IndexBuilder(object):
 
         rc = True
         if self._is_empty_group(make_ndx_output):
-            warnings.warn("Selection produced empty group.%(message)s"
-                          % vars(), category=GromacsValueWarning)
+            warnings.warn("Selection produced empty group.{message!s}".format(**vars()), category=GromacsValueWarning)
             rc = False
         if self._has_syntax_error(make_ndx_output):
             rc = False
@@ -1623,11 +1624,11 @@ class IndexBuilder(object):
 
     def _is_empty_group(self, make_ndx_output):
         m = re.search('Group is empty', make_ndx_output)
-        return not (m is None)
+        return m is not None
 
     def _has_syntax_error(self, make_ndx_output):
         m = re.search('Syntax error:', make_ndx_output)
-        return not (m is None)
+        return m is not None
 
     def __del__(self):
         try:
@@ -1652,7 +1653,7 @@ class Transformer(utilities.FileUtils):
 
     """
 
-    def __init__(self, s="topol.tpr", f="traj.xtc", n=None, force=None, 
+    def __init__(self, s="topol.tpr", f="traj.xtc", n=None, force=None,
                  dirname=os.path.curdir, outdir=None):
         """Set up Transformer with structure and trajectory.
 
@@ -1671,9 +1672,11 @@ class Transformer(utilities.FileUtils):
               groups this should contain those groups)
            *force*
               Set the default behaviour for handling existing files:
-                - ``True``: overwrite existing trajectories
-                - ``False``: throw a IOError exception
-                - ``None``: skip existing and log a warning [default]
+
+              - ``True``: overwrite existing trajectories
+              - ``False``: throw a IOError exception
+              - ``None``: skip existing and log a warning [default]
+
            *dirname*
               directory in which all operations are performed, relative paths
               are interpreted relative to *dirname* [.]
@@ -1696,13 +1699,13 @@ class Transformer(utilities.FileUtils):
                 if f is None:
                     continue
                 if not os.path.exists(f):
-                    msg = "Possible problem: File %(f)r not found in %(dirname)r." % vars()
+                    msg = "Possible problem: File {f!r} not found in {dirname!r}.".format(**vars())
                     warnings.warn(msg, category=MissingDataWarning)
                     logger.warn(msg)
         logger.info("%r initialised", self)
 
     def __repr__(self):
-            return "%s(s=%r, f=%r, n=%r, force=%r)" % (self.__class__.__name__,
+            return "{0!s}(s={1!r}, f={2!r}, n={3!r}, force={4!r})".format(self.__class__.__name__,
                                                        self.tpr, self.xtc, self.ndx, self.force)
     def outfile(self, p):
         """Path for an output file.
@@ -1732,7 +1735,8 @@ class Transformer(utilities.FileUtils):
         """Write compact xtc that is fitted to the tpr reference structure.
 
         See :func:`gromacs.cbook.trj_fitandcenter` for details and
-        description of *kwargs*. The most important ones are listed
+        description of *kwargs* (including *input*, *input1*, *n* and
+        *n1* for how to supply custom index groups). The most important ones are listed
         here but in most cases the defaults should work.
 
         :Keywords:
@@ -1761,11 +1765,11 @@ class Transformer(utilities.FileUtils):
         kwargs.setdefault('o', self.outfile(self.infix_filename(None, self.xtc, '_centfit', 'xtc')))
         force = kwargs.pop('force', self.force)
 
-        logger.info("Centering and fitting trajectory %(f)r..." % kwargs)
+        logger.info("Centering and fitting trajectory {f!r}...".format(**kwargs))
         with utilities.in_dir(self.dirname):
             if not self.check_file_exists(kwargs['o'], resolve="indicate", force=force):
                 trj_fitandcenter(**kwargs)
-            logger.info("Centered and fit trajectory: %(o)r." % kwargs)
+            logger.info("Centered and fit trajectory: {o!r}.".format(**kwargs))
         return {'tpr': self.rp(kwargs['s']), 'xtc': self.rp(kwargs['o'])}
 
     def fit(self, xy=False, **kwargs):
@@ -1790,21 +1794,28 @@ class Transformer(utilities.FileUtils):
              Name of the output trajectory. A default name is created.
              If e.g. *dt* = 100  is one of the *kwargs* then the default name includes
              "_dt100ps".
-          *xy* : boolean
+           *xy* : boolean
              If ``True`` then only do a rot+trans fit in the xy plane
              (good for membrane simulations); default is ``False``.
-          *force*
-            ``True``: overwrite existing trajectories
-            ``False``: throw a IOError exception
-            ``None``: skip existing and log a warning [default]
-          *fitgroup*
-            index group to fit on ["backbone"]
-          *kwargs*
+           *force*
+             Override standard behavior (potentially dangerous)
+             - ``True``: overwrite existing trajectories
+             - ``False``: throw a IOError exception
+             - ``None``: skip existing and log a warning [default]
+
+           *fitgroup*
+             index group to fit on ["backbone"]
+
+             .. Note:: If keyword *input* is supplied then it will override
+                       *fitgroup*; *input* = ``[fitgroup, outgroup]``
+           *kwargs*
              kwargs are passed to :func:`~gromacs.cbook.trj_xyfitted`
 
+
         :Returns:
               dictionary with keys *tpr*, *xtc*, which are the names of the
               the new files
+
         """
         kwargs.setdefault('s', self.tpr)
         kwargs.setdefault('n', self.ndx)
@@ -1818,9 +1829,9 @@ class Transformer(utilities.FileUtils):
             fitmode = kwargs.pop('fit', 'rot+trans')  # user can use 'progressive', too
             infix_default = '_fit'
 
-        dt = kwargs.get('dt', None)
+        dt = kwargs.get('dt')
         if dt:
-            infix_default += '_dt%dps' % int(dt)    # dt in ps
+            infix_default += '_dt{0:d}ps'.format(int(dt))    # dt in ps
 
         kwargs.setdefault('o', self.outfile(self.infix_filename(None, self.xtc, infix_default, 'xtc')))
         fitgroup = kwargs.pop('fitgroup', 'backbone')
@@ -1858,6 +1869,13 @@ class Transformer(utilities.FileUtils):
            *compact*
               ``True``: write a compact and centered trajectory
               ``False``: use trajectory as it is [``False``]
+           *centergroup*
+              Index group used for centering ["Protein"]
+
+              .. Note:: If *input* is provided (see below under *kwargs*)
+                        then *centergroup* is ignored and the group for
+                        centering is taken as the first entry in *input*.
+
            *resn*
               Residue name of the water molecules; all these residues are excluded.
            *groupname*
@@ -1896,14 +1914,22 @@ class Transformer(utilities.FileUtils):
 
         if compact:
             TRJCONV = trj_compact
-            _input = kwargs.get('input', ['Protein'])
+            # input overrides centergroup
+            if kwargs.get('centergroup') is not None and 'input' in kwargs:
+                logger.warn("centergroup = %r will be superceded by input[0] = %r", kwargs['centergroup'], kwargs['input'][0])
+            _input = kwargs.get('input', [kwargs.get('centergroup', 'Protein')])
             kwargs['input'] = [_input[0], groupname]  # [center group, write-out selection]
             del _input
+            logger.info("Creating a compact trajectory centered on group %r", kwargs['input'][0])
+            logger.info("Writing %r to the output trajectory", kwargs['input'][1])
         else:
             TRJCONV = gromacs.trjconv
             kwargs['input'] = [groupname]
+            logger.info("Writing %r to the output trajectory (no centering)", kwargs['input'][0])
+        # clean kwargs, only legal arguments for Gromacs tool trjconv should remain
+        kwargs.pop("centergroup", None)
 
-        NOTwater = "! r %(resn)s" % vars()  # make_ndx selection ("not water residues")
+        NOTwater = "! r {resn!s}".format(**vars())  # make_ndx selection ("not water residues")
         with utilities.in_dir(self.dirname):
             # ugly because I cannot break from the block
             if not self.check_file_exists(newxtc, resolve="indicate", force=force):
@@ -1911,14 +1937,19 @@ class Transformer(utilities.FileUtils):
                 B = IndexBuilder(struct=self.tpr, selections=['@'+NOTwater],
                                  ndx=self.ndx, out_ndx=nowater_ndx)
                 B.combine(name_all=groupname, operation="|", defaultgroups=True)
+                logger.debug("Index file for water removal: %r", nowater_ndx)
 
-                logger.info("TPR file without water %(newtpr)r" % vars())
+                logger.info("TPR file without water {newtpr!r}".format(**vars()))
                 gromacs.tpbconv(s=self.tpr, o=newtpr, n=nowater_ndx, input=[groupname])
 
-                logger.info("NDX of the new system %(newndx)r" % vars())
+                logger.info("NDX of the new system %r", newndx)
                 gromacs.make_ndx(f=newtpr, o=newndx, input=['q'], stderr=False, stdout=False)
+                # PROBLEM: If self.ndx contained a custom group required for fitting then we are loosing
+                #          this group here. We could try to merge only this group but it is possible that
+                #          atom indices changed. The only way to solve this is to regenerate the group with
+                #          a selection or only use Gromacs default groups.
 
-                logger.info("Trajectory without water %(newxtc)r" % vars())
+                logger.info("Trajectory without water {newxtc!r}".format(**vars()))
                 kwargs['s'] = self.tpr
                 kwargs['f'] = self.xtc
                 kwargs['n'] = nowater_ndx
@@ -2015,13 +2046,13 @@ class Transformer(utilities.FileUtils):
                                  ndx=self.ndx, out_ndx=selection_ndx)
                 B.combine(name_all=groupname, operation="|", defaultgroups=True)
 
-                logger.info("TPR file containg the protein %(newtpr)r" % vars())
+                logger.info("TPR file containg the protein {newtpr!r}".format(**vars()))
                 gromacs.tpbconv(s=self.tpr, o=newtpr, n=selection_ndx, input=[groupname])
 
-                logger.info("NDX of the new system %(newndx)r" % vars())
+                logger.info("NDX of the new system {newndx!r}".format(**vars()))
                 gromacs.make_ndx(f=newtpr, o=newndx, input=['q'], stderr=False, stdout=False)
 
-                logger.info("Trajectory with only the protein %(newxtc)r" % vars())
+                logger.info("Trajectory with only the protein {newxtc!r}".format(**vars()))
                 kwargs['s'] = self.tpr
                 kwargs['f'] = self.xtc
                 kwargs['n'] = selection_ndx
@@ -2054,14 +2085,22 @@ class Transformer(utilities.FileUtils):
           *strip_input* = ``['Other']``.
 
         - *input* is passed on to :meth:`fit` and can contain the
-          [center_group, fit_group, output_group]
+          ``[center_group, fit_group, output_group]``
+
+        - *fitgroup* is only passed to :meth:`fit` and just contains
+          the group to fit to ("backbone" by default)
+
+          .. warning:: *fitgroup* can only be a Gromacs default group and not
+                       a custom group (because the indices change after stripping)
 
         - By default *fit* = "rot+trans" (and *fit* is passed to :meth:`fit`,
           together with the *xy* = ``False`` keyword)
+
+        .. Note:: The call signature of :meth:`strip_water` is somewhat different from this one.
         """
         kwargs.setdefault('fit', 'rot+trans')
         kw_fit = {}
-        for k in ('xy', 'fit', 'input'):
+        for k in ('xy', 'fit', 'fitgroup', 'input'):
             if k in kwargs:
                 kw_fit[k] = kwargs.pop(k)
 
