@@ -187,22 +187,29 @@ Classes and functions
 .. autofunction:: break_array
 
 """
+from __future__ import with_statement, absolute_import
 
+from six.moves import zip, range
 
-from __future__ import with_statement
-import os, errno
+import os
+import errno
 import re
 import warnings
+from itertools import cycle
+import logging
 
 import numpy
-
-from gromacs import ParseError, MissingDataError, MissingDataWarning, AutoCorrectionWarning
-import gromacs.utilities as utilities
-from gromacs.odict import odict
+import matplotlib.cm, matplotlib.colors
+from matplotlib import pyplot as plt
 
 import numkit.timeseries
 
-import logging
+from gromacs.exceptions import (ParseError, MissingDataError,
+                                MissingDataWarning, AutoCorrectionWarning)
+import gromacs.utilities as utilities
+import gromacs.collections
+
+
 
 
 class XVG(utilities.FileUtils):
@@ -235,14 +242,15 @@ class XVG(utilities.FileUtils):
      - :attr:`~XVG.tc`: correlation time of the data (assuming a simple
        exponential decay of the fluctuations around the mean)
 
-    These attributes are numpy arrays that correspond to the data columns,
-    i.e. :attr:`XVG.array`[1:].
+    These attributes are numpy arrays that correspond to the data columns in
+    :attr:`XVG.array`, i.e. ``XVG.array[1:]``.
 
     .. Note::
 
        - Only simple XY or NXY files are currently supported, *not*
          Grace files that contain multiple data sets separated by '&'.
        - Any kind of formatting (i.e. :program:`xmgrace` commands) is discarded.
+
     """
 
     #: Default extension of XVG files.
@@ -333,7 +341,7 @@ class XVG(utilities.FileUtils):
         with utilities.openany(self.real_filename, 'w') as xvg:
             xvg.write("# xmgrace compatible NXY data file\n"
                       "# Written by gromacs.formats.XVG()\n")
-            xvg.write("# :columns: %r\n" % self.names)
+            xvg.write("# :columns: {0!r}\n".format(self.names))
             for xyy in self.array.T:
                 xyy.tofile(xvg, sep=" ", format="%-8s")  # quick and dirty ascii output...--no compression!
                 xvg.write('\n')
@@ -390,9 +398,8 @@ class XVG(utilities.FileUtils):
 
         .. SeeAlso:: :func:`numkit.timeseries.tcorrel`
         """
-        from gromacs.analysis.collections import Collection
         t = self.array[0,::nstep]
-        r = Collection([numkit.timeseries.tcorrel(t, Y, nstep=1, **kwargs) for Y in self.array[1:,::nstep]])
+        r = gromacs.collections.Collection([numkit.timeseries.tcorrel(t, Y, nstep=1, **kwargs) for Y in self.array[1:,::nstep]])
         return r
 
     def set_correlparameters(self, **kwargs):
@@ -428,7 +435,7 @@ class XVG(utilities.FileUtils):
 
     def _correlprop(self, key, **kwargs):
         kwargs = self.set_correlparameters(**kwargs)
-        if not self.__cache.get('tcorrel',None) or kwargs.pop('force', False):
+        if not self.__cache.get('tcorrel', None) or kwargs.pop('force', False):
             self.__cache['tcorrel'] = self._tcorrel(**kwargs)
         return numpy.array(self.__cache['tcorrel'].get(key).tolist())
 
@@ -478,14 +485,25 @@ class XVG(utilities.FileUtils):
             ncol = None
             for lineno,line in enumerate(xvg):
                 line = line.strip()
-                if line.startswith(('#', '@')) or len(line) == 0:
+                if len(line) == 0:
                     continue
+                if "label" in line and "xaxis" in line:
+                        self.xaxis = line.split('"')[-2]
+                if "label" in line and "yaxis" in line:
+                        self.yaxis = line.split('"')[-2]
+                if line.startswith("@ legend"):
+                                        if not "legend" in self.metadata: self.metadata["legend"] = []
+                                        self.metadata["legend"].append(line.split("legend ")[-1])
+                if line.startswith("@ s") and "subtitle" not in line:
+                                        name = line.split("legend ")[-1].replace('"','').strip()
+                                        self.names.append(name)
+                if line.startswith(('#', '@')) :
+                                        continue
                 if line.startswith('&'):
-                    raise NotImplementedError('%s: Multi-data not supported, only simple NXY format.'
-                                              % self.real_filename)
+                    raise NotImplementedError('{0!s}: Multi-data not supported, only simple NXY format.'.format(self.real_filename))
                 # parse line as floats
                 try:
-                    row = map(float, line.split())
+                    row = [float(el) for el in line.split()]
                 except:
                     if self.permissive:
                         self.logger.warn("%s: SKIPPING unparsable line %d: %r",
@@ -496,13 +514,13 @@ class XVG(utilities.FileUtils):
                                       self.real_filename, lineno+1, line)
                     raise
                 # check for same number of columns as in previous step
-                if not ncol is None and len(row) != ncol:
+                if ncol is not None and len(row) != ncol:
                     if self.permissive:
                         self.logger.warn("%s: SKIPPING line %d with wrong number of columns: %r",
                                          self.real_filename, lineno+1, line)
                         self.corrupted_lineno.append(lineno+1)
                         continue
-                    errmsg = "%s: Wrong number of columns in line %d: %r" % (self.real_filename, lineno+1, line)
+                    errmsg = "{0!s}: Wrong number of columns in line {1:d}: {2!r}".format(self.real_filename, lineno+1, line)
                     self.logger.error(errmsg)
                     raise IOError(errno.ENODATA, errmsg, self.real_filename)
                 # finally: a good line
@@ -519,6 +537,10 @@ class XVG(utilities.FileUtils):
         finally:
             del rows     # try to clean up as well as possible as it can be massively big
 
+    def to_df(self):
+        import pandas as _pd
+        return _pd.DataFrame(self.array.T, columns=[self.xaxis] + (self.names if len(self.names) else [self.yaxis]) , dtype=float)
+
     def set(self, a):
         """Set the *array* data from *a* (i.e. completely replace).
 
@@ -526,6 +548,14 @@ class XVG(utilities.FileUtils):
         """
         self.__array = numpy.asarray(a)
 
+    def _get_colors(self, color, columns):
+        try:
+            cmap = matplotlib.cm.get_cmap(color)
+            colors = cmap(matplotlib.colors.Normalize()(numpy.arange(len(columns[1:]), dtype=float)))
+        except (TypeError, ValueError):
+            colors = cycle(utilities.asiterable(color))
+        return colors
+
     def plot(self, **kwargs):
         """Plot xvg file data.
 
@@ -559,17 +589,20 @@ class XVG(utilities.FileUtils):
                (will be repeated as necessary); or a matplotlib
                colormap (e.g. "jet", see :mod:`matplotlib.cm`). The
                default is to use the :attr:`XVG.default_color_cycle`.
+          *ax*
+               plot into given axes or create new one if ``None`` [``None``]
           *kwargs*
-               All other keyword arguments are passed on to :func:`pylab.plot`.
-        """
-        from itertools import izip, cycle
-        import matplotlib.cm, matplotlib.colors
-        import pylab
+               All other keyword arguments are passed on to :func:`matplotlib.pyplot.plot`.
 
+        :Returns:
+          *ax*
+               axes instance
+        """
         columns = kwargs.pop('columns', Ellipsis)         # slice for everything
         maxpoints = kwargs.pop('maxpoints', self.maxpoints_default)
         transform = kwargs.pop('transform', lambda x: x)  # default is identity transformation
         method = kwargs.pop('method', "mean")
+        ax = kwargs.pop('ax', None)
 
         if columns is Ellipsis or columns is None:
             columns = numpy.arange(self.array.shape[0])
@@ -586,12 +619,10 @@ class XVG(utilities.FileUtils):
         else:
             a = self.array
 
-        color = kwargs.pop('color', self.default_color_cycle)
-        try:
-            cmap = matplotlib.cm.get_cmap(color)
-            colors = cmap(matplotlib.colors.Normalize()(numpy.arange(len(columns[1:]), dtype=float)))
-        except TypeError:
-            colors = cycle(utilities.asiterable(color))
+        colors = self._get_colors(kwargs.pop('color', self.default_color_cycle), columns)
+
+        if ax is None:
+            ax = plt.gca()
 
         # (decimate/smooth o slice o transform)(array)
         a = self.decimate(method, numpy.asarray(transform(a))[columns], maxpoints=maxpoints)
@@ -600,11 +631,12 @@ class XVG(utilities.FileUtils):
         ma = numpy.ma.MaskedArray(a, mask=numpy.logical_not(numpy.isfinite(a)))
 
         # finally plot (each column separately to catch empty sets)
-        for column, color in izip(xrange(1,len(columns)), colors):
+        for column, color in zip(range(1,len(columns)), colors):
             if len(ma[column]) == 0:
-                warnings.warn("No data to plot for column %(column)d" % vars(), category=MissingDataWarning)
+                warnings.warn("No data to plot for column {column:d}".format(**vars()), category=MissingDataWarning)
             kwargs['color'] = color
-            pylab.plot(ma[0], ma[column], **kwargs)   # plot all other columns in parallel
+            ax.plot(ma[0], ma[column], **kwargs)   # plot all other columns in parallel
+        return ax
 
     def plot_coarsened(self, **kwargs):
         """Plot data like :meth:`XVG.plot` with the range of **all** data shown.
@@ -634,9 +666,7 @@ class XVG(utilities.FileUtils):
 
         .. SeeAlso:: :meth:`XVG.plot`, :meth:`XVG.errorbar` and :meth:`XVG.decimate`
         """
-        from itertools import izip, cycle
-        import matplotlib.cm, matplotlib.colors
-
+        ax = kwargs.pop('ax', None)
         columns = kwargs.pop('columns', Ellipsis)         # slice for everything
         if columns is Ellipsis or columns is None:
             columns = numpy.arange(self.array.shape[0])
@@ -644,18 +674,18 @@ class XVG(utilities.FileUtils):
             raise MissingDataError("plot_coarsened() assumes that there is at least one column "
                                    "of data for the abscissa and one or more for the ordinate.")
 
-        color = kwargs.pop('color', self.default_color_cycle)
-        try:
-            cmap = matplotlib.cm.get_cmap(color)
-            colors = cmap(matplotlib.colors.Normalize()(numpy.arange(len(columns[1:]), dtype=float)))
-        except TypeError:
-            colors = cycle(utilities.asiterable(color))
+        colors = self._get_colors(kwargs.pop('color', self.default_color_cycle), columns)
+
+        if ax is None:
+            ax = plt.gca()
 
         t = columns[0]
         kwargs['demean'] = True
-        for column, color in izip(columns[1:], colors):
+        kwargs['ax'] = ax
+        for column, color in zip(columns[1:], colors):
             kwargs['color'] = color
             self.errorbar(columns=[t, column, column], **kwargs)
+        return ax
 
     def errorbar(self, **kwargs):
         """errorbar plot for a single time series with errors.
@@ -695,8 +725,7 @@ class XVG(utilities.FileUtils):
 
            :meth:`XVG.plot` lists keywords common to both methods.
         """
-        import pylab
-
+        ax = kwargs.pop('ax', None)
         color = kwargs.pop('color', 'black')
         filled = kwargs.pop('filled', True)
         fill_alpha = kwargs.pop('fill_alpha', 0.2)
@@ -721,7 +750,7 @@ class XVG(utilities.FileUtils):
         try:
             data = numpy.asarray(transform(self.array))[columns]
         except IndexError:
-            raise MissingDataError("columns %r are not suitable to index the transformed array, possibly not eneough data" % columns)
+            raise MissingDataError("columns {0!r} are not suitable to index the transformed array, possibly not eneough data".format(columns))
         if data.shape[-1] == 0:
             raise MissingDataError("There is no data to be plotted.")
         a = numpy.zeros((data.shape[0], maxpoints), dtype=numpy.float64)
@@ -761,6 +790,9 @@ class XVG(utilities.FileUtils):
             except IndexError:
                 raise TypeError("Either too few columns selected or data does not have a error column")
 
+        if ax is None:
+            ax = plt.gca()
+
         if filled:
             # can only plot dy
             if error_method == "percentile":
@@ -775,7 +807,7 @@ class XVG(utilities.FileUtils):
             else:
                 y1 = Y - kwargs['yerr']
                 y2 = Y + kwargs['yerr']
-            pylab.fill_between(X, y1, y2, color=color, alpha=fill_alpha)
+            ax.fill_between(X, y1, y2, color=color, alpha=fill_alpha)
         else:
             if error_method == "percentile":
                 # errorbars extend to different lengths;
@@ -789,16 +821,18 @@ class XVG(utilities.FileUtils):
                     kwargs['xerr'] = numpy.vstack((X - mlower[0], X + kwargs['xerr']))
                 except (KeyError, IndexError):
                     pass
-            pylab.errorbar(X, Y, **kwargs)
+            ax.errorbar(X, Y, **kwargs)
 
         # clean up args for plot
         for kw in "yerr", "xerr", "capsize", "ecolor", "elinewidth", "fmt":
             kwargs.pop(kw, None)
         kwargs['alpha'] = 1.0
 
-        pylab.plot(X, Y, color=color, **kwargs)
+        ax.plot(X, Y, color=color, **kwargs)
 
-    def decimate(self, method, a, **kwargs):
+        return ax
+
+    def decimate(self, method, a, maxpoints=10000, **kwargs):
         """Decimate data *a* to *maxpoints* using *method*.
 
         If *a* is a 1D array then it is promoted to a (2, N) array
@@ -853,7 +887,6 @@ class XVG(utilities.FileUtils):
             X = numpy.arange(len(a))
             a = numpy.vstack([X, a])
         ny = a.shape[-1]   # assume 1D or 2D array with last dimension varying fastest
-        maxpoints = kwargs.pop("maxpoints", 10000)
         if maxpoints is None or ny <= maxpoints:
             return a
         return methods[method](a, maxpoints, **kwargs)
@@ -1031,22 +1064,27 @@ class XVG(utilities.FileUtils):
 
         """
         warnings.warn("Using undocumented decimate_error() is highly EXPERIMENTAL",
-                      category=LowAccuracyWarning)
-        return self._decimate(numkit.timeseries.error_histogrammed_function, a, maxpoints, **kwargs)
+                      category=gromacs.exceptions.LowAccuracyWarning)
+        return self._decimate(numkit.timeseries.error_histogrammed_function, a,
+                              maxpoints, **kwargs)
 
     def _decimate(self, func, a, maxpoints, **kwargs):
         ny = a.shape[-1]   # assume 2D array with last dimension varying fastest
         out = numpy.zeros((a.shape[0], maxpoints), dtype=float)
 
         t = a[0]
-        for i in xrange(1, a.shape[0]):
+        for i in range(1, a.shape[0]):
             # compute regularised data for each column separately
             out[i], out[0] = func(t, a[i], bins=maxpoints, **kwargs)
 
         if maxpoints == self.maxpoints_default:  # only warn if user did not set maxpoints
+            try:
+                funcname = func.func_name    # Python 2
+            except AttributeError:
+                funcname = func.__name__     # Python 3
             warnings.warn("Plot had %d datapoints > maxpoints = %d; decimated to %d regularly "
                           "spaced points from the histogrammed data with %s()."
-                          % (ny, maxpoints, maxpoints, func.func_name),
+                          % (ny, maxpoints, maxpoints, funcname),
                           category=AutoCorrectionWarning)
         return out
 
@@ -1084,7 +1122,7 @@ class XVG(utilities.FileUtils):
 
         # smoothed
         out[0,:] = a[0]
-        for i in xrange(1, a.shape[0]):
+        for i in range(1, a.shape[0]):
             # process columns because smooth() only handles 1D arrays :-p
             out[i,:] = numkit.timeseries.smooth(a[i], stepsize, window=window)
 
@@ -1142,9 +1180,8 @@ def break_array(a, threshold=numpy.pi, other=None):
     """
     assert len(a.shape) == 1, "Only 1D arrays supported"
 
-    if other is not None:
-        if not a.shape == other.shape:
-            raise ValueError("arrays must be of identical shape")
+    if other is not None and a.shape != other.shape:
+        raise ValueError("arrays must be of identical shape")
 
     # jump occurs after the index in break
     breaks = numpy.where(numpy.abs(numpy.diff(a)) >= threshold)[0]
