@@ -19,9 +19,6 @@ The working paradigm is that template scripts are provided (see
 :data:`gromacs.config.qscriptdir` (by default ``~/.gromacswrapper/qscripts``)
 and they will be picked up before the package-supplied ones.
 
-The :class:`~gromacs.qsub.Manager` handles setup and control of jobs
-in a queuing system on a remote system via :program:`ssh`.
-
 At the moment, some of the functions in :mod:`gromacs.setup` use this module
 but it is fairly independent and could conceivably be used for a wider range of
 projects.
@@ -194,42 +191,22 @@ Classes and functions
 
 .. autodata:: queuing_systems
 
-.. SeeAlso:: :mod:`gromacs.manager` for classes to manage jobs remotely.
-
 """
+from __future__ import absolute_import, with_statement
 
-import os, errno
+import os
+import errno
+from os.path import relpath
 import warnings
 
-import gromacs.config
-import gromacs.cbook
-from gromacs.utilities import asiterable, Timedelta
-from gromacs import AutoCorrectionWarning
+from . import config
+from . import cbook
+from .utilities import asiterable, Timedelta
+from .exceptions import AutoCorrectionWarning
 
 import logging
 logger = logging.getLogger('gromacs.qsub')
 
-try:
-    from os.path import relpath
-except ImportError:
-    # appeared in python 2.6
-    def relpath(path, start=os.path.curdir):
-        """Return a relative version of a path (from posixpath 2.6)"""
-
-        if not path:
-            raise ValueError("no path specified")
-
-        start_list = os.path.abspath(start).split(os.path.sep)
-        path_list = os.path.abspath(path).split(os.path.sep)
-
-        # Work out how much of the filepath is shared by start and path.
-        i = len(os.path.commonprefix([start_list, path_list]))
-
-        rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
-        if not rel_list:
-            return os.path.curdir
-        return os.path.join(*rel_list)
-
 
 class QueuingSystem(object):
     """Class that represents minimum information about a batch submission system."""
@@ -265,7 +242,7 @@ class QueuingSystem(object):
 
     def has_arrays(self):
         """True if known how to do job arrays."""
-        return not self.array_variable is None
+        return self.array_variable is not None
 
     def array_flag(self, directories):
         """Return string to embed the array launching option in the script."""
@@ -291,10 +268,10 @@ class QueuingSystem(object):
             'declare -a jobdirs']
         for i,dirname in enumerate(asiterable(directories)):
             idx = i+1   # job array indices are 1-based
-            lines.append('jobdirs[%(idx)d]=%(dirname)r' % vars())
+            lines.append('jobdirs[{idx:d}]={dirname!r}'.format(**vars()))
         lines.extend([
                 '# Switch to the current tasks directory:',
-                'wdir="${jobdirs[${%(array_variable)s}]}"' % vars(self),
+                'wdir="${{jobdirs[${{{array_variable!s}}}]}}"'.format(**vars(self)),
                 'cd "$wdir" || { echo "ERROR: failed to enter $wdir."; exit 1; }',
                 hrule,
                 ''
@@ -316,6 +293,7 @@ queuing_systems = [
     QueuingSystem('Sun Gridengine', 'sge', '#$', array_variable='SGE_TASK_ID', array_option='-t %d-%d'),
     QueuingSystem('PBS', 'pbs', '#PBS', array_variable='PBS_ARRAY_INDEX', array_option='-J %d-%d'),
     QueuingSystem('LoadLeveler', 'll', '#@'),  # no idea how to do arrays in LL
+    QueuingSystem('Slurm', 'slu', '#SBATCH'), # will add array settings
     ]
 
 def detect_queuing_system(scriptfile):
@@ -370,12 +348,12 @@ def generate_submit_scripts(templates, prefix=None, deffnm='md', jobname='MD', b
     """
     if not jobname[0].isalpha():
         jobname = 'MD_'+jobname
-        wmsg = "To make the jobname legal it must start with a letter: changed to %r" % jobname
+        wmsg = "To make the jobname legal it must start with a letter: changed to {0!r}".format(jobname)
         logger.warn(wmsg)
         warnings.warn(wmsg, category=AutoCorrectionWarning)
     if prefix is None:
         prefix = ""
-    if not mdrun_opts is None:
+    if mdrun_opts is not None:
         mdrun_opts = '"'+str(mdrun_opts)+'"'  # TODO: could test if quotes already present
 
     dirname = kwargs.pop('dirname', os.path.curdir)
@@ -386,34 +364,52 @@ def generate_submit_scripts(templates, prefix=None, deffnm='md', jobname='MD', b
 
     def write_script(template):
         submitscript = os.path.join(dirname, prefix + os.path.basename(template))
-        logger.info("Setting up queuing system script %(submitscript)r..." % vars())
+        logger.info("Setting up queuing system script {submitscript!r}...".format(**vars()))
         # These substitution rules are documented for the user in the module doc string
-        gromacs.cbook.edit_txt(template,
-                               [('^ *DEFFNM=','(?<==)(.*)', deffnm),
-                                ('^#.*(-N|job_name)', '((?<=-N\s)|(?<=job_name\s))\s*\w+', jobname),
-                                ('^#.*(-A|account_no)', '((?<=-A\s)|(?<=account_no\s))\s*\w+', budget),
-                                ('^#.*(-l walltime|wall_clock_limit)', '(?<==)(\d+:\d+:\d+)', walltime),
-                                ('^ *WALL_HOURS=', '(?<==)(.*)', wall_hours),
-                                ('^ *STARTDIR=', '(?<==)(.*)', startdir),
-                                ('^ *NPME=', '(?<==)(.*)', npme),
-                                ('^ *MDRUN_OPTS=', '(?<==)("")', mdrun_opts),  # only replace literal ""
-                                ('^# JOB_ARRAY_PLACEHOLDER', '^.*$', jobarray_string),
-                                ],
-                               newname=submitscript)
-        ext = os.path.splitext(submitscript)[1]
+        qsystem = detect_queuing_system(template)
+        if qsystem is not None and (qsystem.name == 'Slurm'):
+            cbook.edit_txt(template,
+                           [('^ *DEFFNM=','(?<==)(.*)', deffnm),
+                            ('^#.*(-J)', '((?<=-J\s))\s*\w+', jobname),
+                            ('^#.*(-A|account_no)', '((?<=-A\s)|(?<=account_no\s))\s*\w+', budget),
+                            ('^#.*(-t)', '(?<=-t\s)(\d+:\d+:\d+)', walltime),
+                            ('^ *WALL_HOURS=', '(?<==)(.*)', wall_hours),
+                            ('^ *STARTDIR=', '(?<==)(.*)', startdir),
+                            ('^ *NPME=', '(?<==)(.*)', npme),
+                            ('^ *MDRUN_OPTS=', '(?<==)("")', mdrun_opts),  # only replace literal ""
+                            ('^# JOB_ARRAY_PLACEHOLDER', '^.*$', jobarray_string),
+                            ],
+                           newname=submitscript)
+            ext = os.path.splitext(submitscript)[1]
+        else:
+            cbook.edit_txt(template,
+                           [('^ *DEFFNM=','(?<==)(.*)', deffnm),
+                            ('^#.*(-N|job_name)', '((?<=-N\s)|(?<=job_name\s))\s*\w+', jobname),
+                            ('^#.*(-A|account_no)', '((?<=-A\s)|(?<=account_no\s))\s*\w+', budget),
+                            ('^#.*(-l walltime|wall_clock_limit)', '(?<==)(\d+:\d+:\d+)', walltime),
+                            ('^ *WALL_HOURS=', '(?<==)(.*)', wall_hours),
+                            ('^ *STARTDIR=', '(?<==)(.*)', startdir),
+                            ('^ *NPME=', '(?<==)(.*)', npme),
+                            ('^ *MDRUN_OPTS=', '(?<==)("")', mdrun_opts),  # only replace literal ""
+                            ('^# JOB_ARRAY_PLACEHOLDER', '^.*$', jobarray_string),
+                           ],
+                           newname=submitscript)
+            ext = os.path.splitext(submitscript)[1]
         if ext in ('.sh', '.csh', '.bash'):
-            os.chmod(submitscript, 0755)
+            os.chmod(submitscript, 0o755)
         return submitscript
 
-    return [write_script(template) for template in gromacs.config.get_templates(templates)]
+    return [write_script(template) for template in config.get_templates(templates)]
 
 
 def generate_submit_array(templates, directories, **kwargs):
     """Generate a array job.
 
     For each ``work_dir`` in *directories*, the array job will
-     1. cd into ``work_dir``
-     2. run the job as detailed in the template
+
+    1. cd into ``work_dir``
+    2. run the job as detailed in the template
+
     It will use all the queuing system directives found in the
     template. If more complicated set ups are required, then this
     function cannot be used.
@@ -422,7 +418,9 @@ def generate_submit_array(templates, directories, **kwargs):
        *templates*
           Basic template for a single job; the job array logic is spliced into
           the position of the line ::
+
               # JOB_ARRAY_PLACEHOLDER
+
           The appropriate commands for common queuing systems (Sun Gridengine, PBS)
           are hard coded here. The queuing system is detected from the suffix of
           the template.
@@ -452,5 +450,4 @@ def generate_submit_array(templates, directories, **kwargs):
         return generate_submit_scripts(template, **kwargs)[0]   # returns list of length 1
 
     # must use config.get_templates() because we need to access the file for detecting
-    return [write_script(template) for template in gromacs.config.get_templates(templates)]
-
+    return [write_script(template) for template in config.get_templates(templates)]
