@@ -1,4 +1,4 @@
-# GromacsWrapper: cbook.py
+# GromacsWrapper: setup.py
 """
 :mod:`gromacs.setup` -- Setting up a Gromacs MD run
 ===================================================
@@ -115,7 +115,7 @@ Defined constants:
 
 """
 
-from __future__ import with_statement
+from __future__ import absolute_import, with_statement
 
 __docformat__ = "restructuredtext en"
 
@@ -130,14 +130,15 @@ logger = logging.getLogger('gromacs.setup')
 
 
 import gromacs
-import gromacs.config as config
-from gromacs import GromacsError, GromacsFailureWarning, GromacsValueWarning, \
-     AutoCorrectionWarning, BadParameterWarning, UsageWarning, MissingDataError
-import gromacs.cbook
-from gromacs.cbook import add_mdp_includes
-import gromacs.qsub
-import gromacs.utilities
-from gromacs.utilities import in_dir, realpath, Timedelta, asiterable, firstof
+from . import config
+from .exceptions import (GromacsError, GromacsFailureWarning, GromacsValueWarning,
+                         AutoCorrectionWarning, BadParameterWarning, UsageWarning,
+                         MissingDataError)
+from . import run
+from . import cbook
+from . import qsub
+from . import utilities
+from .utilities import in_dir, realpath, Timedelta, asiterable, firstof
 
 
 #: Concentration of water at standard conditions in mol/L.
@@ -159,19 +160,19 @@ rlist        1.4 ?      1.0
 """
 
 
-
 trj_compact_main = gromacs.tools.Trjconv(ur='compact', center=True, boxcenter='tric', pbc='mol',
-                                         input=('__main__','system'),
-                                         doc="Returns a compact representation of the system centered on the __main__ group")
-
+                                         input=('__main__', 'system'))
+# trj_compact_main.__doc__ += "Returns a compact representation of the system centered on the __main__ group"
 
 # TODO:
 # - should be part of a class so that we can store the topology etc !!!
 #   and also store mainselection
 
 def topology(struct=None, protein='protein',
-             top='system.top',  dirname='top', 
-             posres="posres.itp", **pdb2gmx_args):
+             top='system.top',  dirname='top',
+             posres="posres.itp",
+             ff="oplsaa", water="tip4p",
+             **pdb2gmx_args):
     """Build Gromacs topology files from pdb.
 
     :Keywords:
@@ -183,8 +184,13 @@ def topology(struct=None, protein='protein',
            name of the topology file
        *dirname*
            directory in which the new topology will be stored
+       *ff*
+           force field (string understood by ``pdb2gmx``); default
+           "oplsaa"
+       *water*
+           water model (string), default "tip4p"
        *pdb2gmxargs*
-           arguments for ``pdb2gmx`` such as ``ff``, ``water``, ...
+           other arguments for ``pdb2gmx``
 
     .. note::
        At the moment this function simply runs ``pdb2gmx`` and uses
@@ -200,10 +206,11 @@ def topology(struct=None, protein='protein',
     if posres is None:
         posres = protein + '_posres.itp'
 
-    pdb2gmx_args.update({'f': structure, 'o': new_struct, 'p': top, 'i': posres})
+    pdb2gmx_args.update({'f': structure, 'o': new_struct, 'p': top, 'i': posres,
+                         'ff': ff, 'water': water})
 
     with in_dir(dirname):
-        logger.info("[%(dirname)s] Building topology %(top)r from struct = %(struct)r" % vars())
+        logger.info("[{dirname!s}] Building topology {top!r} from struct = {struct!r}".format(**vars()))
         # perhaps parse output from pdb2gmx 4.5.x to get the names of the chain itp files?
         gromacs.pdb2gmx(**pdb2gmx_args)
     return { \
@@ -246,38 +253,38 @@ def make_main_index(struct, selection='"Protein"', ndx='main.ndx', oldndx=None):
     added later as could be other symbolic groups such as __membrane__.
     """
 
-    logger.info("Building the main index file %(ndx)r..." % vars())
+    logger.info("Building the main index file {ndx!r}...".format(**vars()))
 
     # pass 1: select
     # get a list of groups
     # need the first "" to get make_ndx to spit out the group list.
     _,out,_ = gromacs.make_ndx(f=struct, n=oldndx, o=ndx, stdout=False,
                                       input=("", "q"))
-    groups = gromacs.cbook.parse_ndxlist(out)
-    
-    # find the matching groups, 
+    groups = cbook.parse_ndxlist(out)
+
+    # find the matching groups,
     # there is a nasty bug in GROMACS where make_ndx may have multiple
-    # groups, which caused the previous approach to fail big time. 
+    # groups, which caused the previous approach to fail big time.
     # this is a work around the make_ndx bug.
-    # striping the "" allows compatibility with existing make_ndx selection commands. 
+    # striping the "" allows compatibility with existing make_ndx selection commands.
     selection = selection.strip("\"")
-    
+
     selected_groups = [g for g in groups if g['name'].lower() == selection.lower()]
-    
+
     if len(selected_groups) > 1:
         logging.warn("make_ndx created duplicated groups, performing work around")
-        
+
     if len(selected_groups) <= 0:
-        msg = "no groups found for selection {}, available groups are {}".format(selection, groups)
+        msg = "no groups found for selection {0}, available groups are {1}".format(selection, groups)
         logging.error(msg)
         raise ValueError(msg)
-    
+
     # Found at least one matching group, we're OK
-        
-    # index of last group    
-    last = len(groups) - 1               
+
+    # index of last group
+    last = len(groups) - 1
     assert last == groups[-1]['nr']
-    
+
     group = selected_groups[0]
 
     # pass 2:
@@ -285,19 +292,19 @@ def make_main_index(struct, selection='"Protein"', ndx='main.ndx', oldndx=None):
     # 2) __environment__ is everything else (eg SOL, ions, ...)
     _,out,_ = gromacs.make_ndx(f=struct, n=ndx, o=ndx,
                                       stdout=False,
-                                             # make copy selected group, this now has index last + 1 
-                                      input=("{}".format(group['nr']),
+                                             # make copy selected group, this now has index last + 1
+                                      input=("{0}".format(group['nr']),
                                              # rename this to __main__
-                                             "name {} __main__".format(last+1),
+                                             "name {0} __main__".format(last+1),
                                              # make a complement to this group, it get index last + 2
-                                             "! \"__main__\"",  
+                                             "! \"__main__\"",
                                              # rename this to __environment__
-                                             "name {} __environment__".format(last+2),
+                                             "name {0} __environment__".format(last+2),
                                              # list the groups
                                              "",
-                                             # quit 
+                                             # quit
                                              "q"))
-    return gromacs.cbook.parse_ndxlist(out)
+    return cbook.parse_ndxlist(out)
 
 
 #: Hard-coded lipid residue names for a ``vdwradii.dat`` file. Use together with
@@ -315,10 +322,10 @@ def get_lipid_vdwradii(outdir=os.path.curdir, libdir=None):
     """
     vdwradii_dat = os.path.join(outdir, "vdwradii.dat")
 
-    if not libdir is None:
+    if libdir is not None:
         filename = os.path.join(libdir, 'vdwradii.dat')  # canonical name
         if not os.path.exists(filename):
-            msg = 'No VDW database file found in %(filename)r.' % vars()
+            msg = 'No VDW database file found in {filename!r}.'.format(**vars())
             logger.exception(msg)
             raise OSError(msg, errno.ENOENT)
     else:
@@ -326,35 +333,209 @@ def get_lipid_vdwradii(outdir=os.path.curdir, libdir=None):
             filename = os.path.join(os.environ['GMXLIB'], 'vdwradii.dat')
         except KeyError:
             try:
-                filename = os.path.join(os.environ['GMXDATA'], 'gromacs', 'top', 'vdwradii.dat')
+                filename = os.path.join(os.environ['GMXDATA'], 'top', 'vdwradii.dat')
             except KeyError:
-                msg = "Cannot find vdwradii.dat. Set GMXLIB or GMXDATA."
+                msg = "Cannot find vdwradii.dat. Set GMXLIB (point to 'top') or GMXDATA ('share/gromacs')."
                 logger.exception(msg)
                 raise OSError(msg, errno.ENOENT)
     if not os.path.exists(filename):
-        msg = "Cannot find %r; something is wrong with the Gromacs installation." % vars()
+        msg = "Cannot find {filename!r}; something is wrong with the Gromacs installation.".format(**vars())
         logger.exception(msg, errno.ENOENT)
         raise OSError(msg)
 
     # make sure to catch 3 and 4 letter resnames
-    patterns = vdw_lipid_resnames + list(set([x[:3] for x in vdw_lipid_resnames]))
+    patterns = vdw_lipid_resnames + list({x[:3] for x in vdw_lipid_resnames})
     # TODO: should do a tempfile...
     with open(vdwradii_dat, 'w') as outfile:
         # write lipid stuff before general
         outfile.write('; Special larger vdw radii for solvating lipid membranes\n')
         for resname in patterns:
             for atom,radius in vdw_lipid_atom_radii.items():
-                outfile.write('%(resname)4s %(atom)-5s %(radius)5.3f\n' % vars())
+                outfile.write('{resname:4!s} {atom:<5!s} {radius:5.3f}\n'.format(**vars()))
         with open(filename, 'r') as infile:
             for line in infile:
                 outfile.write(line)
-    logger.debug('Created lipid vdW radii file %(vdwradii_dat)r.' % vars())
+    logger.debug('Created lipid vdW radii file {vdwradii_dat!r}.'.format(**vars()))
     return realpath(vdwradii_dat)
 
+def solvate_sol(struct='top/protein.pdb', top='top/system.top',
+                distance=0.9, boxtype='dodecahedron',
+                water='tip4p', solvent_name='SOL', with_membrane=False,
+                dirname='solvate',
+                **kwargs):
+    structure = realpath(struct)
+    topology = realpath(top)
+
+    # arguments for editconf that we honour
+    editconf_keywords = ["box", "bt", "angles", "c", "center", "aligncenter",
+                         "align", "translate", "rotate", "princ"]
+    editconf_kwargs = dict((k,kwargs.pop(k,None)) for k in editconf_keywords)
+    editconf_boxtypes = ["triclinic", "cubic", "dodecahedron", "octahedron", None]
+
+    # needed for topology scrubbing
+    scrubber_kwargs = {'marker': kwargs.pop('marker',None)}
+
+    # sanity checks and argument dependencies
+    bt = editconf_kwargs.pop('bt')
+    boxtype = bt if bt else boxtype   # bt takes precedence over boxtype
+    if not boxtype in editconf_boxtypes:
+        msg = "Unsupported boxtype {boxtype!r}: Only {boxtypes!r} are possible.".format(**vars())
+        logger.error(msg)
+        raise ValueError(msg)
+    if editconf_kwargs['box']:
+        distance = None    # if box is set then user knows what she is doing...
+
+    if water.lower() in ('spc', 'spce'):
+        water = 'spc216'
+    elif water.lower() == 'tip3p':
+        water = 'spc216'
+        logger.warning("TIP3P water model selected: using SPC equilibrated box "
+                       "for initial solvation because it is a reasonable starting point "
+                       "for any 3-point model. EQUILIBRATE THOROUGHLY!")
+
+    # clean topology (if user added the marker; the default marker is
+    # ; Gromacs auto-generated entries follow:
+    n_removed = cbook.remove_molecules_from_topology(topology, **scrubber_kwargs)
+
+    with in_dir(dirname):
+        logger.info("[{dirname!s}] Solvating with water {water!r}...".format(**vars()))
+        if boxtype is None:
+            hasBox = False
+            ext = os.path.splitext(structure)[1]
+            if ext == '.gro':
+                hasBox = True
+            elif ext == '.pdb':
+                with open(structure) as struct:
+                    for line in struct:
+                        if line.startswith('CRYST'):
+                            hasBox = True
+                            break
+            if not hasBox:
+                msg = "No box data in the input structure {structure!r} and boxtype is set to None".format(**vars())
+                logger.exception(msg)
+                raise MissingDataError(msg)
+            distance = boxtype = None   # ensures that editconf just converts
+        editconf_kwargs.update({'f': structure, 'o': 'boxed.gro',
+                                'bt': boxtype, 'd': distance})
+        gromacs.editconf(**editconf_kwargs)
+
+        if with_membrane:
+            vdwradii_dat = get_lipid_vdwradii()  # need to clean up afterwards
+            logger.info("Using special vdW radii for lipids {0!r}".format(vdw_lipid_resnames))
+
+        try:
+            gromacs.genbox(p=topology, cp='boxed.gro', cs=water, o='solvated.gro')
+        except:
+            if with_membrane:
+                # remove so that it's not picked up accidentally
+                utilities.unlink_f(vdwradii_dat)
+            raise
+        logger.info("Solvated system with %s", water)
+    return {'struct': realpath(dirname, 'solvated.gro'),}
+
+def solvate_ion(struct='solvated.gro', top='top/system.top',
+                concentration=0, cation='NA', anion='CL',
+                solvent_name='SOL', ndx='main.ndx',
+                mainselection='"Protein"', dirname='solvate',
+                **kwargs):
+    structure = realpath(struct)
+    topology = realpath(top)
+    # By default, grompp should not choke on a few warnings because at
+    # this stage the user cannot do much about it (can be set to any
+    # value but is kept undocumented...)
+    grompp_maxwarn = kwargs.pop('maxwarn',10)
+
+    # handle additional include directories (kwargs are also modified!)
+    mdp_kwargs = cbook.add_mdp_includes(topology, kwargs)
+
+    with in_dir(dirname):
+        with open('none.mdp','w') as mdp:
+            mdp.write('; empty mdp file\ninclude = {include!s}\nrcoulomb = 1\nrvdw = 1\nrlist = 1\n'.format(**mdp_kwargs))
+        qtotgmx = cbook.grompp_qtot(f='none.mdp', o='topol.tpr', c=structure,
+                                    p=topology, stdout=False, maxwarn=grompp_maxwarn)
+        qtot = round(qtotgmx)
+        logger.info("[{dirname!s}] After solvation: total charge qtot = {qtotgmx!r} = {qtot!r}".format(**vars()))
+
+        if concentration != 0:
+            logger.info("[{dirname!s}] Adding ions for c = {concentration:f} M...".format(**vars()))
+            # target concentration of free ions c ==>
+            #    N = N_water * c/c_water
+            # add ions for concentration to the counter ions (counter ions are less free)
+            #
+            # get number of waters (count OW ... works for SPC*, TIP*P water models)
+            rc,output,junk = gromacs.make_ndx(f='topol.tpr', o='ow.ndx',
+                                              input=('keep 0', 'del 0', 'a OW*', 'name 0 OW', '', 'q'),
+                                              stdout=False)
+            groups = cbook.parse_ndxlist(output)
+            gdict = {g['name']: g for g in groups}   # overkill...
+            N_water = gdict['OW']['natoms']                  # ... but dict lookup is nice
+            N_ions = int(N_water * concentration/CONC_WATER) # number of monovalents
+        else:
+            N_ions = 0
+
+        # neutralize (or try -neutral switch of genion???)
+        n_cation = n_anion = 0
+        if qtot > 0:
+            n_anion = int(abs(qtot))
+        elif qtot < 0:
+            n_cation = int(abs(qtot))
+
+        n_cation += N_ions
+        n_anion  += N_ions
+
+        if n_cation != 0 or n_anion != 0:
+            # sanity check:
+            assert qtot + n_cation - n_anion < 1e-6
+            logger.info("[{dirname!s}] Adding n_cation = {n_cation:d} and n_anion = {n_anion:d} ions...".format(**vars()))
+            gromacs.genion(s='topol.tpr', o='ionized.gro', p=topology,
+                           pname=cation, nname=anion, np=n_cation, nn=n_anion,
+                           input=solvent_name)
+        else:
+            # fake ionized file ... makes it easier to continue without too much fuzz
+            try:
+                os.unlink('ionized.gro')
+            except OSError as err:
+                if err.errno != errno.ENOENT:
+                    raise
+            os.symlink('solvated.gro', 'ionized.gro')
+
+        qtot = cbook.grompp_qtot(f='none.mdp', o='ionized.tpr', c='ionized.gro',
+                                 p=topology, stdout=False, maxwarn=grompp_maxwarn)
+
+        if abs(qtot) > 1e-4:
+            wmsg = "System has non-zero total charge qtot = {qtot:g} e.".format(**vars())
+            warnings.warn(wmsg, category=BadParameterWarning)
+            logger.warn(wmsg)
+
+        # make main index
+        try:
+            make_main_index('ionized.tpr', selection=mainselection, ndx=ndx)
+        except GromacsError as err:
+            # or should I rather fail here?
+            wmsg = "Failed to make main index file %r ... maybe set mainselection='...'.\n"\
+                   "The error message was:\n%s\n" % (ndx, str(err))
+            logger.warn(wmsg)
+            warnings.warn(wmsg, category=GromacsFailureWarning)
+        try:
+            trj_compact_main(f='ionized.gro', s='ionized.tpr', o='compact.pdb', n=ndx)
+        except GromacsError as err:
+            wmsg = "Failed to make compact pdb for visualization... pressing on regardless. "\
+                   "The error message was:\n%s\n" % str(err)
+            logger.warn(wmsg)
+            warnings.warn(wmsg, category=GromacsFailureWarning)
+
+    return {'qtot': qtot,
+            'struct': realpath(dirname, 'ionized.gro'),
+            'ndx': realpath(dirname, ndx),      # not sure why this is propagated-is it used?
+            'mainselection': mainselection,
+            }
+
+
+
 def solvate(struct='top/protein.pdb', top='top/system.top',
             distance=0.9, boxtype='dodecahedron',
             concentration=0, cation='NA', anion='CL',
-            water='spc', solvent_name='SOL', with_membrane=False,
+            water='tip4p', solvent_name='SOL', with_membrane=False,
             ndx = 'main.ndx', mainselection = '"Protein"',
             dirname='solvate',
             **kwargs):
@@ -432,163 +613,18 @@ def solvate(struct='top/protein.pdb', top='top/system.top',
           changed in the mdp file.
 
     """
-    structure = realpath(struct)
-    topology = realpath(top)
+    sol = solvate_sol(struct=struct, top=top,
+                      distance=distance, boxtype=boxtype,
+                      water=water, solvent_name=solvent_name,
+                      with_membrane=with_membrane,
+                      dirname=dirname, **kwargs)
 
-    # arguments for editconf that we honour
-    editconf_keywords = ["box", "bt", "angles", "c", "center", "aligncenter",
-                         "align", "translate", "rotate", "princ"]
-    editconf_kwargs = dict((k,kwargs.pop(k,None)) for k in editconf_keywords)
-    editconf_boxtypes = ["triclinic", "cubic", "dodecahedron", "octahedron", None]
-
-    # needed for topology scrubbing
-    scrubber_kwargs = {'marker': kwargs.pop('marker',None)}
-
-    # sanity checks and argument dependencies
-    bt = editconf_kwargs.pop('bt')
-    boxtype = bt if bt else boxtype   # bt takes precedence over boxtype
-    if not boxtype in editconf_boxtypes:
-        msg = "Unsupported boxtype %(boxtype)r: Only %(boxtypes)r are possible." % vars()
-        logger.error(msg)
-        raise ValueError(msg)
-    if editconf_kwargs['box']:
-        distance = None    # if box is set then user knows what she is doing...
-
-    # handle additional include directories (kwargs are also modified!)
-    mdp_kwargs = add_mdp_includes(topology, kwargs)
-
-    if water.lower() in ('spc', 'spce'):
-        water = 'spc216'
-    elif water.lower() == 'tip3p':
-        water = 'spc216'
-        logger.warning("TIP3P water model selected: using SPC equilibrated box "
-                       "for initial solvation because it is a reasonable starting point "
-                       "for any 3-point model. EQUILIBRATE THOROUGHLY!")
-
-    # By default, grompp should not choke on a few warnings because at
-    # this stage the user cannot do much about it (can be set to any
-    # value but is kept undocumented...)
-    grompp_maxwarn = kwargs.pop('maxwarn',10)
-
-    # clean topology (if user added the marker; the default marker is
-    # ; Gromacs auto-generated entries follow:
-    n_removed = gromacs.cbook.remove_molecules_from_topology(topology, **scrubber_kwargs)
-
-    with in_dir(dirname):
-        logger.info("[%(dirname)s] Solvating with water %(water)r..." % vars())
-        if boxtype is None:
-            hasBox = False
-            ext = os.path.splitext(structure)[1]
-            if ext == '.gro':
-                hasBox = True
-            elif ext == '.pdb':
-                with open(structure) as struct:
-                    for line in struct:
-                        if line.startswith('CRYST'):
-                            hasBox = True
-                            break
-            if not hasBox:
-                msg = "No box data in the input structure %(structure)r and boxtype is set to None" % vars()
-                logger.exception(msg)
-                raise MissingDataError(msg)
-            distance = boxtype = None   # ensures that editconf just converts
-        editconf_kwargs.update({'f': structure, 'o': 'boxed.gro',
-                                'bt': boxtype, 'd': distance})
-        gromacs.editconf(**editconf_kwargs)
-
-        if with_membrane:
-            vdwradii_dat = get_lipid_vdwradii()  # need to clean up afterwards
-            logger.info("Using special vdW radii for lipids %r" % vdw_lipid_resnames)
-
-        try:
-            gromacs.genbox(p=topology, cp='boxed.gro', cs=water, o='solvated.gro')
-        except:
-            if with_membrane:
-                # remove so that it's not picked up accidentally
-                gromacs.utilities.unlink_f(vdwradii_dat)
-            raise
-        logger.info("Solvated system with %s", water)
-
-        with open('none.mdp','w') as mdp:
-            mdp.write('; empty mdp file\ninclude = %(include)s\nrcoulomb = 1\nrvdw = 1\nrlist = 1\n' % mdp_kwargs)
-        qtotgmx = gromacs.cbook.grompp_qtot(f='none.mdp', o='topol.tpr', c='solvated.gro',
-                                            p=topology, stdout=False, maxwarn=grompp_maxwarn)
-        qtot = round(qtotgmx)
-        logger.info("[%(dirname)s] After solvation: total charge qtot = %(qtotgmx)r = %(qtot)r" % vars())
-
-        if concentration != 0:
-            logger.info("[%(dirname)s] Adding ions for c = %(concentration)f M..." % vars())
-            # target concentration of free ions c ==>
-            #    N = N_water * c/c_water
-            # add ions for concentration to the counter ions (counter ions are less free)
-            #
-            # get number of waters (count OW ... works for SPC*, TIP*P water models)
-            rc,output,junk = gromacs.make_ndx(f='topol.tpr', o='ow.ndx',
-                                              input=('keep 0', 'del 0', 'a OW*', 'name 0 OW', '', 'q'),
-                                              stdout=False)
-            groups = gromacs.cbook.parse_ndxlist(output)
-            gdict = dict([(g['name'], g) for g in groups])   # overkill...
-            N_water = gdict['OW']['natoms']                  # ... but dict lookup is nice
-            N_ions = int(N_water * concentration/CONC_WATER) # number of monovalents
-        else:
-            N_ions = 0
-
-        # neutralize (or try -neutral switch of genion???)
-        n_cation = n_anion = 0
-        if qtot > 0:
-            n_anion = int(abs(qtot))
-        elif qtot < 0:
-            n_cation = int(abs(qtot))
-
-        n_cation += N_ions
-        n_anion  += N_ions
-
-        if n_cation != 0 or n_anion != 0:
-            # sanity check:
-            assert qtot + n_cation - n_anion < 1e-6
-            logger.info("[%(dirname)s] Adding n_cation = %(n_cation)d and n_anion = %(n_anion)d ions..." % vars())
-            gromacs.genion(s='topol.tpr', o='ionized.gro', p=topology,
-                           pname=cation, nname=anion, np=n_cation, nn=n_anion,
-                           input=solvent_name)
-        else:
-            # fake ionized file ... makes it easier to continue without too much fuzz
-            try:
-                os.unlink('ionized.gro')
-            except OSError, err:
-                if err.errno != errno.ENOENT:
-                    raise
-            os.symlink('solvated.gro', 'ionized.gro')
-
-        qtot = gromacs.cbook.grompp_qtot(f='none.mdp', o='ionized.tpr', c='ionized.gro',
-                                         p=topology, stdout=False, maxwarn=grompp_maxwarn)
-
-        if abs(qtot) > 1e-4:
-            wmsg = "System has non-zero total charge qtot = %(qtot)g e." % vars()
-            warnings.warn(wmsg, category=BadParameterWarning)
-            logger.warn(wmsg)
-
-        # make main index
-        try:
-            make_main_index('ionized.tpr', selection=mainselection, ndx=ndx)
-        except GromacsError, err:
-            # or should I rather fail here?
-            wmsg = "Failed to make main index file %r ... maybe set mainselection='...'.\n"\
-                   "The error message was:\n%s\n" % (ndx, str(err))
-            logger.warn(wmsg)
-            warnings.warn(wmsg, category=GromacsFailureWarning)
-        try:
-            trj_compact_main(f='ionized.gro', s='ionized.tpr', o='compact.pdb', n=ndx)
-        except GromacsError, err:
-            wmsg = "Failed to make compact pdb for visualization... pressing on regardless. "\
-                   "The error message was:\n%s\n" % str(err)
-            logger.warn(wmsg)
-            warnings.warn(wmsg, category=GromacsFailureWarning)
-
-    return {'qtot': qtot,
-            'struct': realpath(dirname, 'ionized.gro'),
-            'ndx': realpath(dirname, ndx),      # not sure why this is propagated-is it used?
-            'mainselection': mainselection,
-            }
+    ion = solvate_ion(struct=sol['struct'], top=top,
+                      concentration=concentration, cation=cation, anion=anion,
+                      solvent_name=solvent_name, ndx=ndx,
+                      mainselection=mainselection, dirname=dirname,
+                      **kwargs)
+    return ion
 
 
 def check_mdpargs(d):
@@ -602,7 +638,8 @@ def check_mdpargs(d):
 def energy_minimize(dirname='em', mdp=config.templates['em.mdp'],
                     struct='solvate/ionized.gro', top='top/system.top',
                     output='em.pdb', deffnm="em",
-                    mdrunner=None, **kwargs):
+                    mdrunner=None, mdrun_args=None,
+                    **kwargs):
     """Energy minimize the system.
 
     This sets up the system (creates run input files) and also runs
@@ -632,6 +669,12 @@ def energy_minimize(dirname='em', mdp=config.templates['em.mdp'],
           just try :func:`gromacs.mdrun_d` and :func:`gromacs.mdrun` but a
           MDrunner instance gives the user the ability to run mpi jobs
           etc. [None]
+       *mdrun_args*
+          arguments for *mdrunner* (as a dict), e.g. ``{'nt': 2}``;
+          empty by default
+
+          .. versionadded:: 0.7.0
+
        *kwargs*
           remaining key/value pairs that should be changed in the
           template mdp file, eg ``nstxtcout=250, nstfout=250``.
@@ -645,6 +688,8 @@ def energy_minimize(dirname='em', mdp=config.templates['em.mdp'],
     mdp_template = config.get_template(mdp)
     deffnm = deffnm.strip()
 
+    mdrun_args = {} if mdrun_args is None else mdrun_args
+
     # write the processed topology to the default output
     kwargs.setdefault('pp', 'processed.top')
 
@@ -660,36 +705,30 @@ def energy_minimize(dirname='em', mdp=config.templates['em.mdp'],
     # only interesting when passed from solvate()
     qtot = kwargs.pop('qtot', 0)
 
+    # mdp is now the *output* MDP that will be generated from mdp_template
     mdp = deffnm+'.mdp'
     tpr = deffnm+'.tpr'
 
-    logger.info("[%(dirname)s] Energy minimization of struct=%(struct)r, top=%(top)r, mdp=%(mdp)r ..." % vars())
+    logger.info("[{dirname!s}] Energy minimization of struct={struct!r}, top={top!r}, mdp={mdp!r} ...".format(**vars()))
 
-    add_mdp_includes(topology, kwargs)
+    cbook.add_mdp_includes(topology, kwargs)
 
     if qtot != 0:
         # At the moment this is purely user-reported and really only here because
         # it might get fed into the function when using the keyword-expansion pipeline
         # usage paradigm.
-        wmsg = "Total charge was reported as qtot = %(qtot)g <> 0; probably a problem." % vars()
+        wmsg = "Total charge was reported as qtot = {qtot:g} <> 0; probably a problem.".format(**vars())
         logger.warn(wmsg)
         warnings.warn(wmsg, category=BadParameterWarning)
 
     with in_dir(dirname):
-        unprocessed = gromacs.cbook.edit_mdp(mdp_template, new_mdp=mdp, **kwargs)
+        unprocessed = cbook.edit_mdp(mdp_template, new_mdp=mdp, **kwargs)
         check_mdpargs(unprocessed)
-        gromacs.grompp(f=mdp, o=tpr, c=structure, p=topology, **unprocessed)
-        mdrun_args = dict(v=True, stepout=10, deffnm=deffnm, c=output)
+        gromacs.grompp(f=mdp, o=tpr, c=structure, r=structure, p=topology, **unprocessed)
+        mdrun_args.update(v=True, stepout=10, deffnm=deffnm, c=output)
         if mdrunner is None:
-            try:
-                gromacs.mdrun_d(**mdrun_args)
-            except (AttributeError, OSError):
-                # fall back to mdrun if no double precision binary
-                wmsg = "No 'mdrun_d' binary found so trying 'mdrun' instead.\n"\
-                    "(Note that energy minimization runs better with mdrun_d.)"
-                logger.warn(wmsg)
-                warnings.warn(wmsg, category=AutoCorrectionWarning)
-                gromacs.mdrun(**mdrun_args)
+            mdrun = run.get_double_or_single_prec_mdrun()
+            mdrun(**mdrun_args)
         else:
             if type(mdrunner) is type:
                 # class
@@ -713,7 +752,7 @@ def energy_minimize(dirname='em', mdp=config.templates['em.mdp'],
             raise GromacsError(errmsg)
         final_struct = realpath(output)
 
-    logger.info("[%(dirname)s] energy minimized structure %(final_struct)r" % vars())
+    logger.info("[{dirname!s}] energy minimized structure {final_struct!r}".format(**vars()))
     return {'struct': final_struct,
             'top': topology,
             'mainselection': mainselection,
@@ -737,9 +776,11 @@ def em_schedule(**kwargs):
 
     :Example:
        Conduct three minimizations:
-         1. low memory Broyden-Goldfarb-Fletcher-Shannon (BFGS) for 30 steps
-         2. steepest descent for 200 steps
-         3. finish with BFGS for another 30 steps
+
+       1. low memory Broyden-Goldfarb-Fletcher-Shannon (BFGS) for 30 steps
+       2. steepest descent for 200 steps
+       3. finish with BFGS for another 30 steps
+
        We also do a multi-processor minimization when possible (i.e. for steep
        (and conjugate gradient) by using a :class:`gromacs.run.MDrunner` class
        for a :program:`mdrun` executable compiled for OpenMP in 64 bit (see
@@ -761,7 +802,7 @@ def em_schedule(**kwargs):
     kwargs.pop('integrator', None)  # clean input; we set intgerator from integrators
     nsteps = kwargs.pop('nsteps', [100, 1000])
 
-    outputs = ['em%03d_%s.pdb' % (i,integrator) for i,integrator in enumerate(integrators)]
+    outputs = ['em{0:03d}_{1!s}.pdb'.format(i, integrator) for i,integrator in enumerate(integrators)]
     outputs[-1] = kwargs.pop('output', 'em.pdb')
 
     files = {'struct': kwargs.pop('struct', None)}  # fake output from energy_minimize()
@@ -820,17 +861,18 @@ def _setup_MD(dirname,
     mdp_parameters = {'nsteps':nsteps, 'dt':dt, 'pp': 'processed.top'}
     mdp_parameters.update(mdp_kwargs)
 
-    add_mdp_includes(topology, mdp_parameters)
+    cbook.add_mdp_includes(topology, mdp_parameters)
 
+    logger.info("[%(dirname)s] input mdp  = %(mdp_template)r", vars())
     with in_dir(dirname):
         if not (mdp_parameters.get('Tcoupl','').lower() == 'no' or mainselection is None):
-            logger.info("[%(dirname)s] Automatic adjustment of T-coupling groups" % vars())
+            logger.info("[{dirname!s}] Automatic adjustment of T-coupling groups".format(**vars()))
 
             # make index file in almost all cases; with mainselection == None the user
             # takes FULL control and also has to provide the template or index
             groups = make_main_index(structure, selection=mainselection,
                                      oldndx=index, ndx=mainindex)
-            natoms = dict([(g['name'], float(g['natoms'])) for g in groups])
+            natoms = {g['name']: float(g['natoms']) for g in groups}
             tc_group_names = ('__main__', '__environment__')   # defined in make_main_index()
             try:
                 x = natoms['__main__']/natoms['__environment__']
@@ -895,14 +937,18 @@ def _setup_MD(dirname,
             mdp_parameters['ref_p'] = ""
             mdp_parameters['compressibility'] = ""
 
-        unprocessed = gromacs.cbook.edit_mdp(mdp_template, new_mdp=mdp, **mdp_parameters)
+        unprocessed = cbook.edit_mdp(mdp_template, new_mdp=mdp, **mdp_parameters)
         check_mdpargs(unprocessed)
         gromacs.grompp(f=mdp, p=topology, c=structure, n=index, o=tpr, **unprocessed)
 
-        runscripts = gromacs.qsub.generate_submit_scripts(
+        runscripts = qsub.generate_submit_scripts(
             qscript_template, deffnm=deffnm, jobname=qname, budget=budget,
             startdir=startdir, mdrun_opts=mdrun_opts, walltime=walltime)
 
+    logger.info("[%(dirname)s] output mdp = %(mdp)r", vars())
+    logger.info("[%(dirname)s] output ndx = %(ndx)r", vars())
+    logger.info("[%(dirname)s] output tpr = %(tpr)r", vars())
+    logger.info("[%(dirname)s] output runscripts = %(runscripts)r", vars())
     logger.info("[%(dirname)s] All files set up for a run time of %(runtime)g ps "
                 "(dt=%(dt)g, nsteps=%(nsteps)g)" % vars())
 
@@ -993,7 +1039,7 @@ def MD_restrained(dirname='MD_POSRES', **kwargs):
     .. _`pressure coupling`: http://manual.gromacs.org/online/mdp_opt.html#pc
     """
 
-    logger.info("[%(dirname)s] Setting up MD with position restraints..." % vars())
+    logger.info("[{dirname!s}] Setting up MD with position restraints...".format(**vars()))
     kwargs.setdefault('struct', 'em/em.pdb')
     kwargs.setdefault('qname', 'PR_GMX')
     kwargs.setdefault('define', '-DPOSRES')
@@ -1071,7 +1117,7 @@ def MD(dirname='MD', **kwargs):
               change the *define* parameter in the mdp file)
     """
 
-    logger.info("[%(dirname)s] Setting up MD..." % vars())
+    logger.info("[{dirname!s}] Setting up MD...".format(**vars()))
     kwargs.setdefault('struct', 'MD_POSRES/md.gro')
     kwargs.setdefault('qname', 'MD_GMX')
     return _setup_MD(dirname, **kwargs)
